@@ -16,6 +16,7 @@ struct _Ipc_Instance
    const char *icon_name;
    const char *font;
    const char *startup_id;
+   const char *startup_split;
    int x, y, w, h;
    int pos;
    int login_shell;
@@ -26,6 +27,8 @@ struct _Ipc_Instance
    int maximized;
    int hold;
    int nowm;
+   int xterm_256color;
+   int active_links;
 };
 
 void ipc_init(void);
