@@ -5,26 +5,18 @@
 #include "options_elm.h"
 
 static void
-launch_elm_config(void *data EINA_UNUSED,
-                Evas_Object *obj EINA_UNUSED,
-                void *event_info EINA_UNUSED)
+launch_elm_config(void *_data EINA_UNUSED,
+                Evas_Object *_obj EINA_UNUSED,
+                void *_event_info EINA_UNUSED)
 {
-   int pid;
+   Ecore_Exe *exe;
 
-   pid = fork();
-   if (pid < 0)
-     {
-        ERR("unable to fork: %s", strerror(errno));
-        return;
-     }
-   if (pid == 0)
-     {
-        execl("/bin/sh", "sh", "-c", "elementary_config", NULL);
-     }
+   exe = ecore_exe_pipe_run("elementary_config", ECORE_EXE_NONE, NULL);
+   ecore_exe_free(exe);
 }
 
 void
-options_elm(Evas_Object *opbox, Evas_Object *term EINA_UNUSED)
+options_elm(Evas_Object *opbox, Evas_Object *_term EINA_UNUSED)
 {
    Evas_Object *o, *fr, *bx, *bt, *en;
 
