@@ -39,7 +39,8 @@ Eina_Bool    termio_cwd_get(const Evas_Object *obj, char *buf, size_t size);
 Evas_Object *termio_textgrid_get(const Evas_Object *obj);
 Evas_Object *termio_win_get(const Evas_Object *obj);
 const char  *termio_title_get(const Evas_Object *obj);
-void         termio_title_set(Evas_Object *obj, const char *title);
+const char  *termio_user_title_get(const Evas_Object *obj);
+void         termio_user_title_set(Evas_Object *obj, const char *title);
 const char  *termio_icon_name_get(const Evas_Object *obj);
 void         termio_media_mute_set(Evas_Object *obj, Eina_Bool mute);
 void         termio_media_visualize_set(Evas_Object *obj, Eina_Bool visualize);
