@@ -1,11 +1,11 @@
-#include "private.h"
+#include "coverity.h"
+
 #include <stdio.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-
-#include <Eina.h>
+#include "private.h"
 #include "tycommon.h"
 
 static void
@@ -31,7 +31,7 @@ main(int argc, char **argv)
      {
         char tbuf[32];
         snprintf(tbuf, sizeof(tbuf), "%c}bt", 0x1b);
-        if (write(0, tbuf, strlen(tbuf) + 1) != (signed)(strlen(tbuf) + 1)) perror("write");
+        if (write(1, tbuf, strlen(tbuf) + 1) != (signed)(strlen(tbuf) + 1)) perror("write");
         return 0;
      }
    for (i = 1; i < argc; i++)
@@ -50,7 +50,7 @@ main(int argc, char **argv)
           snprintf(tbuf, sizeof(tbuf), "%c}bp%s", 0x1b, path);
         else
           snprintf(tbuf, sizeof(tbuf), "%c}bt%s", 0x1b, path);
-        if (write(0, tbuf, strlen(tbuf) + 1) != (signed)(strlen(tbuf) + 1)) perror("write");
+        if (write(1, tbuf, strlen(tbuf) + 1) != (signed)(strlen(tbuf) + 1)) perror("write");
      }
    return 0;
 }
