@@ -10,6 +10,7 @@ typedef struct _Term  Term;
 
 Eina_Bool main_term_popup_exists(const Term *term);
 void term_unfocus(Term *term);
+void term_focus(Term *term);
 
 Evas_Object *term_termio_get(const Term *term);
 Evas_Object *term_miniview_get(const Term *term);
@@ -47,6 +48,11 @@ Config *win_config_get(const Win *wn);
 void win_term_swallow(Win *wn, Term *term);
 void win_add_split(Win *wn, Term *term);
 void win_sizing_handle(Win *wn);
+void win_toggle_visible_group(Win *wn);
+void win_toggle_all_group(Win *wn);
+Eina_Bool win_is_group_action_handled(Win *wn);
+Eina_Bool win_is_group_input(Win *wn);
+
 
 void term_ref(Term *term);
 void term_unref(Term *term);
@@ -61,6 +67,13 @@ void term_down(Term *term);
 void term_left(Term *term);
 void term_right(Term *term);
 
+const char *
+term_preedit_str_get(Term *term);
+Ecore_IMF_Context *
+term_imf_context_get(Term *term);
+
+Eina_Bool term_is_visible(Term *term);
+
 void win_font_size_set(Win *wn, int new_size);
 void win_font_update(Term *term);
 
@@ -74,5 +87,8 @@ typedef Eina_Bool (*For_Each_Term)(Term *term, void *data);
 Eina_Bool
 for_each_term_do(Win *wn, For_Each_Term cb, void *data);
 
+void
+term_apply_shine(Term *term, int shine);
+void background_set_shine(Config *config, Evas_Object *bg);
 
 #endif
